/***********************************************
Dummy GUI c++ classes for DS
**********
Code under GPL license.
(c)Thoduv (thoduv@free.fr) 2006
http://thoduv.drunkencoders.com/
************************************************/

typedef struct { u16 x, y;	} Point;
typedef struct {	u16 x, y, w, h;	} Rect;

typedef enum
{
	ALIGN_RIGHT,
	ALIGN_CENTER
} TextAlignement;

typedef enum
{
 	COLOR_NORMAL,
	COLOR_INVERT,
	COLOR_SELECTED,
	COLOR_DISABLED	
} TextColor;

class Keyboard
{
	public:

		typedef void (*chgVScrollFn)(int scrollValue);
		typedef enum
		{
			SCROLLED_IN,
			SCROLLING_IN,
			SCROLLED_OUT,
			SCROLLING_OUT
		} ScrollStatus;
		enum
		{
			KEY_SHIFT = 1,
			KEY_CAPS = 2,
			KEY_CANCEL = 3,
			KEY_ESCAPE = 4,
			KEY_CTRL = 5,
			KEY_ALT = 6,
			KEY_WINDOWS = 7,
			KEY_F1 = 11,
			KEY_F2 = 12,
			KEY_F3 = 13,
			KEY_F4 = 14,
			KEY_F5 = 15,
			KEY_F6 = 16,
			KEY_F7 = 17,
			KEY_F8 = 18,
			KEY_F9 = 19,
			KEY_F10 = 20,
			KEY_F11 = 21,
			KEY_F12 = 22,
		};
		enum
		{
			KB_HEIGHT = 16,
			KB_WIDTH = 32
		};
		typedef const char Shape[KB_HEIGHT][KB_WIDTH];
		
		Keyboard(u8 *bitmapTiles, u16 mw, const u8 *kbBitmap, chgVScrollFn chgVScroll, Shape *kbShape) : bitmapTiles(bitmapTiles), mw(mw), kbBitmap(kbBitmap), kbShape(kbShape), scrollSpeed(6), chgVScroll(chgVScroll), vscrollValue(192), vscrollStatus(SCROLLED_OUT)
		{
			chgVScroll(vscrollValue);
		}
		~Keyboard() { }

		void draw()
		{
			char k;
			u16 x, y;
			Shape &curShape = kbShape[kbMode];
			blitBitmapOnTiles8(bitmapTiles, kbBitmap, KB_WIDTH*8, KB_HEIGHT*8);
			for(y=0; y<KB_HEIGHT; y++)
			{
				for(x=0; x<KB_WIDTH; x++)
				{
					k = curShape[y][x];
					if(!k) continue;
					Rect kr = findKey(k);
#define putMonoCharHere(c) monofont_putChar(bitmapTiles, plotOnTiles8, (kr.x<<3)+1 +((kr.w<<2)-4), (kr.y<<3)+((kr.h<<2)-5), MONOFONT_DEFAULTFONT, 254, 0, c);
#define putMonoStrHere(s) monofont_printTextInRect(bitmapTiles, plotOnTiles8, (kr.x<<3), (kr.y<<3), (kr.w<<3), (kr.h<<3), MONOFONT_DEFAULTFONT, 254, 0, s);
					switch(k)
					{
						case KEY_ESCAPE: putMonoStrHere("Es");
							break;
						case KEY_F1: putMonoStrHere("F1");
							break;
						case KEY_F2: putMonoStrHere("F2");
							break;
						case KEY_F3: putMonoStrHere("F3");
							break;
						case KEY_F4: putMonoStrHere("F4");
							break;
						case KEY_F5: putMonoStrHere("F5");
							break;
						case KEY_F6: putMonoStrHere("F6");
							break;
						case KEY_F7: putMonoStrHere("F7");
							break;
						case KEY_F8: putMonoStrHere("F8");
							break;
						case KEY_F9: putMonoStrHere("F9");
							break;
						case KEY_F10: putMonoStrHere("F10");
							break;
						case KEY_F11: putMonoStrHere("F11");
							break;
						case KEY_F12: putMonoStrHere("F12");
							break;
						case KEY_SHIFT: putMonoStrHere("Shf");
							break;
						case KEY_CAPS: putMonoStrHere("Cp");
							break;
						case KEY_CANCEL: putMonoStrHere("<");
							break;
						case '\n': putMonoStrHere("Ent");
							break;
						case '\x8': putMonoStrHere("Del");
							break;
						default: putMonoCharHere(k);
							break;
					}
#undef putMonoCharHere
#undef putMonoStrHere
					x += kr.w - 1;
				}
			}
		}
		void scrollUpdate()
		{
			if(vscrollStatus == SCROLLING_IN)
			{
				vscrollValue -= scrollSpeed;
				if(vscrollValue <= (192-(KB_HEIGHT*8))) { vscrollValue = (192-(KB_HEIGHT*8)); vscrollStatus = SCROLLED_IN; }
				chgVScroll(vscrollValue);
			}
			else if(vscrollStatus == SCROLLING_OUT)
			{
				vscrollValue += scrollSpeed;
				if(vscrollValue >= 192) { vscrollValue = 192; vscrollStatus = SCROLLED_OUT; }
				chgVScroll(vscrollValue);
			}
		}
		void scrollIn() { if(vscrollStatus != SCROLLED_IN) { vscrollStatus = SCROLLING_IN; } }
		void scrollOut() { if(vscrollStatus != SCROLLED_OUT) { vscrollStatus = SCROLLING_OUT; } }
		ScrollStatus scrollStatus() { return vscrollStatus; }
		
		char touch(Point rp)
		{
			if(vscrollStatus != SCROLLED_IN) { return 0; }
			Point p = {rp.x >> 3, (rp.y - vscrollValue) >> 3};
			char k = kbShape[kbMode][p.y][p.x];
			switch(k)
			{
				case KEY_SHIFT:
					if(kbMode == MODE_SHIFT) { changeMode(MODE_NORMAL); }
					else { changeMode(MODE_SHIFT); }
					return 0;
				case KEY_CAPS:
					if(kbMode == MODE_CAPS) { changeMode(MODE_NORMAL); }
					else { changeMode(MODE_CAPS); }
					return 0;
				default:
					if(kbMode == MODE_SHIFT) { changeMode(MODE_NORMAL); }
					return k;
			}
		}
		
		bool isInside(Point p) { return p.y > vscrollValue; }
		
	protected:
		typedef enum
		{
			MODE_NORMAL = 0,
			MODE_SHIFT = 1,
			MODE_CAPS = 2
		} Mode;
		
		Rect findKey(char k)
		{
			Shape &curShape = kbShape[kbMode];
			Rect kr = {0, 0, 0, 0};
			int x, y;
			for(y=0; y<KB_HEIGHT; y++) { for(x=0; x<KB_WIDTH; x++) { if(curShape[y][x] == k) { kr.x = x; kr.y = y-1; break; } } }
			while(curShape[kr.y][kr.x+kr.w] == k) { kr.w++; }
			while(curShape[kr.y+kr.h][kr.x] == k) { kr.h++; }
			return kr;
		}
		
		void changeMode(Mode mode)
		{
			kbMode = mode;
			draw();
		}
		
		u8 *bitmapTiles;
		u16 mw;
		const u8 *kbBitmap;
		Shape *kbShape;
		Mode kbMode;
		
		u16 scrollSpeed;
		chgVScrollFn chgVScroll;
		int vscrollValue;
		ScrollStatus vscrollStatus;
};

class Drawable
{
	public:
		Drawable(u16 *map, u16 *text, u16 mw) : map(map), text(text), mw(mw) { } 
		~Drawable() { }
		void clear()
		{
			u16 i;
			if(!cleared)
			{
				for(i = 0; i < mw * 32; i++) map[i] = 0;
				for(i = 0; i < (mw << 3) * 192 / 4; i++) text[i] = 0x0000;
				cleared = true;
			}
		}
		void clearTextRect(Rect r)
		{
			u16 x,y;
			for(y = 0; y < r.h; y++) { for(x = 0; x < r.w; x++) { plotOnTiles4((u8*)text, mw, x+r.x, y+r.y, 0); } }
		}
		void putOnMap(Point p, u16 val) { map[p.y * mw + p.x] = val;  cleared = false; }
		void drawBoxOnMap(Rect r, u16 tl, u16 tm, u16 tr, u16 ml, u16 mm, u16 mr, u16 bl, u16 bm, u16 br)
		{
			u16 *pmap = map + r.y * mw + r.x;
			u16 cx,cy;
			pmap[0] = tl;
			for(cx = 0; cx < r.w; cx++) pmap[cx+1] = tm;
			pmap[r.w+1] = tr;
			for(cy=0; cy < r.h ; cy++)
			{
				pmap[(cy+1)*mw] = ml;
				for(cx = 0; cx < r.w; cx++) pmap[((cy+1)*mw)+cx+1] = mm;
				pmap[(cy+1)*mw+r.w+1] = mr;
			}
			pmap[(r.h+1)*mw] = bl;
			for(cx = 0; cx < r.w; cx++) pmap[(r.h+1)*mw+cx+1] = bm;
			pmap[(r.h+1)*mw+r.w+1] = br;
			
			cleared = false;
		}
		void drawText(Rect r, TextAlignement align, TextColor color, const char *str)
		{
			u8 pColor;
			switch(color)
			{
				case COLOR_INVERT: pColor = 3; break;
				case COLOR_SELECTED: pColor = 2; break;
				default: pColor = 1; break;
			}
			switch(align)
			{
				case ALIGN_CENTER: monofont_printTextInRect((u8*)text, plotOnTiles4, r.x, r.y, r.w, r.h, MONOFONT_DEFAULTFONT, pColor, 0, str);
					break;
				case ALIGN_RIGHT: monofont_printText((u8*)text, plotOnTiles4, r.x + 8, r.y + r.h / 2  -4, r.w, r.h, MONOFONT_DEFAULTFONT, pColor, 0, str);
					break;
			}
			
			cleared = false;
		}
	protected:
		bool cleared;
		u16 *map;
		u16 *text;
		u16 mw;
};

class Box
{
	public:

		typedef struct
		{
			u16 tl, tm, tr, ml, mm, mr, bl, bm, br;
		} Gfx;
		
		Box(Drawable *d, Rect p, Gfx g)	: d(d), rect(p), gfx(g)
		{
			caption[0] = '\0';
			computeRect();
		}
		virtual ~Box() { }
		virtual void putCaption(TextAlignement align, const char *c) { strncpy(caption, c, 256); captionAlign = align; }
		virtual const char *getCaption() { return caption; }
		virtual void draw()
		{
			d->drawBoxOnMap(rect, gfx.tl, gfx.tm, gfx.tr, gfx.ml, gfx.mm, gfx.mr, gfx.bl, gfx.bm, gfx.br);
			d->clearTextRect(r_rect);
			d->drawText(r_rect, captionAlign, COLOR_NORMAL, caption);
		}
		virtual bool isInside(Point p) { return (p.x>r_rect.x) && (p.x<r_rect.x+r_rect.w) && (p.y>r_rect.y) && (p.y<r_rect.y+r_rect.h); }
		
	protected:
		void computeRect()
		{
			r_rect.x = rect.x << 3;
			r_rect.y = rect.y << 3;
			r_rect.w = (rect.w+2) << 3;
			r_rect.h = (rect.h+2) << 3;
		}	
		Drawable *d;
		Rect rect;
		Rect r_rect;
		Gfx gfx;
		char caption[256];
		TextAlignement captionAlign;
};

class ScrollBox : public Box
{
	public:
		typedef struct
		{
			u16 stl, str, sbl, sbr, rl, rr;
		} Gfx;
		
		ScrollBox(Drawable *d, Rect p, Box::Gfx g, Gfx sg) : Box(d, p, g), gfx(sg), selected(-1)
		{
			u16 i;
			for(i=0; i<32; i++) { list[i] = NULL; }
		}
		virtual ~ScrollBox() { }
		
		void draw()
		{
			int i;
			Box::draw();
			
			d->putOnMap((Point){rect.x + rect.w + 2, rect.y}, gfx.stl);
			d->putOnMap((Point){rect.x + rect.w + 3, rect.y}, gfx.str);
			d->putOnMap((Point){rect.x + rect.w + 2, rect.y + 1}, gfx.sbl);
			d->putOnMap((Point){rect.x + rect.w + 3, rect.y + 1}, gfx.sbr);
			for(i=0; i < (rect.h-2); i++)
			{
				d->putOnMap((Point){rect.x + rect.w + 2, rect.y + i + 2}, gfx.rl | (1 << 11));
				d->putOnMap((Point){rect.x + rect.w + 3, rect.y + i + 2}, gfx.rr | (1 << 11));
			}
			d->putOnMap((Point){rect.x + rect.w + 2, rect.y + rect.h}, gfx.sbl | (1 << 11));
			d->putOnMap((Point){rect.x + rect.w + 3, rect.y + rect.h}, gfx.sbr | (1 << 11));
			d->putOnMap((Point){rect.x + rect.w + 2, rect.y + rect.h + 1}, gfx.stl | (1 << 11));
			d->putOnMap((Point){rect.x + rect.w + 3, rect.y + rect.h + 1}, gfx.str | (1 << 11));	
			
			u16 p = 0;
			for(i=0; i<32; i++)
			{
				if(list[i])
				{
					d->drawText((Rect){r_rect.x + 4, r_rect.y + 4 + (p * 16), r_rect.w - 8, 16}, ALIGN_RIGHT, (selected == i) ? COLOR_SELECTED : COLOR_NORMAL, list[i]);
					p++;
				}
			}
		}	
		
		int addString(const char *c)
		{
			u16 i;
			for(i=0; i<32; i++) { if(!list[i]) { list[i] = strdup(c); return i;	} }
			return -1;
		}
		void remString(u16 id)
		{
			if(id < 32)
			{
				free(list[id]); list[id] = NULL;
				if(id == selected) { selected == -1; }
			}
		}
		int getSelected()	{ return selected; }
		void select(int i) { selected = i; }
		
		void touch(Point p)
		{
			if(!isInside(p)) return;
			Point rp = {p.x-r_rect.x, p.y-r_rect.y};
			int newSelected = ((rp.y - 8) / 16);
			if(list[newSelected]) { selected = newSelected; draw(); }
		}
		
	protected:
		Gfx gfx;
		
		int selected;
		char *list[32];
};
 
class InputBox : public Box
{
	public:
		InputBox(Drawable *d, Rect p, Box::Gfx g) : Box(d, p, g) { }
		virtual ~InputBox() { }
		
		void handleInput(char k)
		{
			u16 capSize = strlen(caption);
			switch(k)
			{
				case 0: return;
				case '\x8':
					if(capSize > 0) { caption[capSize-1] = '\0'; }
					break;
				default:
					caption[capSize] = k;
					caption[capSize+1] = '\0';
					break;
			}
			draw();
		}
	protected:
};

class CheckBox : public Box
{
	public:
		
		CheckBox(Drawable *d, Rect p, Box::Gfx g, Box::Gfx cg) : Box(d, p, g), checked(false), normalGfx(g), checkedGfx(cg) { }
		virtual ~CheckBox() { }
		
		void draw()
		{
			if(checked) d->drawBoxOnMap((Rect){rect.x, rect.y, 1, 1}, checkedGfx.tl, checkedGfx.tm, checkedGfx.tr, checkedGfx.ml, checkedGfx.mm, checkedGfx.mr, checkedGfx.bl, checkedGfx.bm, checkedGfx.br);
			else d->drawBoxOnMap((Rect){rect.x, rect.y, 1, 1}, normalGfx.tl, normalGfx.tm, normalGfx.tr, normalGfx.ml, normalGfx.mm, normalGfx.mr, normalGfx.bl, normalGfx.bm, normalGfx.br);
			
			d->drawText((Rect){r_rect.x + (3*8), r_rect.y, r_rect.w-(3*8), r_rect.h}, captionAlign, COLOR_NORMAL, caption);
		}
		
		void touch(Point p)
		{
			if(isInside(p))
			{
				checked = !checked;
				draw();
			}
		}
		
		bool isChecked() { return checked; }
		void setCheck(bool c) { checked = c; }
		
	protected:
		bool checked;
		Box::Gfx normalGfx;
		Box::Gfx checkedGfx;
};



