/*
 *  Copyright (C) 2006 Thoduv.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#ifndef __MONOFONT_H_
#define __MONOFONT_H_

#include <nds/jtypes.h>

#define MONOFONT_OVERWRITE		1
#define MONOFONT_ONLYCOMPUTE	2

#define MONOFONT_NOLIMIT		~0

typedef void (monofont_fnPlot)(u8*, u16, u16, u16, u16);

typedef struct
{
	u8 xSize;
	u8 ySize;
	u16 monoData[16];	/* Donnes monochrome 1bit/pixel : 16*16 */
} __attribute__((__packed__)) monofont_char;

typedef struct
{
	int x;
	int w;
} monofont_return;

/* Taille des espaces, peut-tre modifie */
extern u16 monofont_spaceSize;

/* Police par dfaut de Monofont,  compiler sparement */
#define MONOFONT_DEFAULTFONT monofont_default
extern const monofont_char monofont_default[256];

#define monofont_flagChar		'#'
#define monofont_flagColor		"#c"

#define monofont_char8printText(b,x,y,mx,my,p,c,f,t) monofont_printText(b, &ZdsVideo_char8Plot, x, y,mx,my, p, c, f,t)
#define monofont_bmp8printText(b,x,y,mx,my,p,c,f,t) monofont_printText(b, &ZdsVideo_bmp8Plot, x, y,mx,my, p, c, f,t)
#define monofont_char4printText(b,x,y,mx,my,p,c,f,t) monofont_printText(b, &ZdsVideo_char4Plot, x, y,mx,my, p, c, f,t)

extern const monofont_char *monofont_putChar(u8 *bmpVram, monofont_fnPlot *fnPlot, u16 x, u16 y, const monofont_char *police, u16 textColor, u16 monofontFlags, char c);
extern int monofont_printText(u8 *bmpVram, monofont_fnPlot *fnPlot, u16 x, u16 y, u16 mx, u16 my, const monofont_char *police, u16 textColor, u16 monofontFlags, const char *text);
extern int monofont_printTextInRect(u8 *bmpVram, monofont_fnPlot *fnPlot, u16 x, u16 y, u16 w, u16 h, const monofont_char *police, u16 textColor, u16 monofontFlags, const char *text);

#define monofont_bmp8printTextCenter(b,x,y,mx,my,p,c,f,t) monofont_printTextCenter(b, &ZdsVideo_bmp8Plot, x, y,mx,my, p, c, f,t)
#define monofont_char4printTextCenter(b,x,y,mx,my,p,c,f,t) monofont_printTextCenter(b, &ZdsVideo_char4Plot, x, y,mx,my, p, c, f,t)

#define monofont_computeLineSize(p,l)	monofont_printText(NULL, NULL, 0, 0, MONOFONT_NOLIMIT, MONOFONT_NOLIMIT, p, 0, MONOFONT_ONLYCOMPUTE, l);
extern monofont_return monofont_printTextCenter(u8 *bmpVram, monofont_fnPlot *fnPlot, u16 x, u16 y, u16 mx, u16 my, const monofont_char *police, u16 textColor, u16 monofontFlags, const char *text);

//////////// fonction plottantes //////////////

extern void plotOnTiles4(u8 *charblock, u16 mw, u16 x, u16 y, u16 color);
extern void plotOnTiles8(u8 *charblock, u16 mw, u16 x, u16 y, u16 color);
extern void plotOnBmp16(u8 *charblock, u16 mw, u16 x, u16 y, u16 color);


#endif /*__MONOFONT_H_*/
