#include <SDL/SDL.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include "VncClient.h"

void SocketSetBlocking(int fd, bool block)
{
	fcntl(fd, F_SETFL, O_NONBLOCK);
}

void handleError(VncClient *client)
{
	char error[256];
	switch(client->catchError(error))
	{
		case VncClient::ERROR_FATAL:
			fprintf(stderr, "VncClient fatal: %s\n", error);
			SDL_Quit();
			exit(-1);
			break;
		case VncClient::ERROR_WARNING:
			fprintf(stderr, "VncClient warning: %s\n", error);
			break;
		default:
			break;
	}
}

SDL_Surface *mainSurface;
SDL_Surface *vncSurface;
SDL_Surface *conversionSurface;

void handleFb(VncClient *client, const void *raw, uint32_t size)
{
	static uint32_t p = 0;
	uint32_t i;
	
	fprintf(stderr, "       handleFb\n");
	
	SDL_LockSurface(vncSurface);
	for(i = 0; i < size; i++) { ((char*)vncSurface->pixels)[i+p] = ((char*)raw)[i]; }
	SDL_UnlockSurface(vncSurface);
	
	p+=size;
}


int main(int argc, char *argv[])
{
	SDL_Rect zeroRect;
	SDL_Event event;
	bool mainLoop;
	VncClient *client;
	
	if(argc != 2) { fprintf(stderr, "Usage: %s vnc-server\n", argv[0]); exit(0); }
	
	SDL_Init(SDL_INIT_VIDEO);
	SDL_WM_SetCaption("VncClient", NULL);
		
	client = new VncClient(&handleFb, &handleError);
	
	client->connect(argv[1], 5900);
	client->handshake();
	
	VncClient::Rect wholeScreen;
	client->getRemoteResolution(&wholeScreen);
	
	mainSurface = SDL_SetVideoMode(wholeScreen.w, wholeScreen.h, 32, SDL_HWSURFACE);
	vncSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, wholeScreen.w, wholeScreen.h, 32, 0, 0, 0, 0);
	
	char title[256];
	sprintf(title, "VncClient: %s", client->getRemoteName());
	SDL_WM_SetCaption(title, NULL);
	
	VncClient::PixelFormat pixelFormat;
	
	pixelFormat.depth = 16;
	pixelFormat.redMax = 0xff;
	pixelFormat.greenMax = 0xff;
	pixelFormat.blueMax = 0xff;
	pixelFormat.redShift = 0;
	pixelFormat.blueShift = 8;
	pixelFormat.greenShift = 16;
	
// 	client->setPixelFormat(&pixelFormat);
	client->setEncodings();
	
	client->framebufferUpdateRequest(&wholeScreen, false);
	
	zeroRect.x = 0;
	zeroRect.y = 0;
	
	mainLoop = true;
	while(mainLoop)
	{
		
		client->update();
		
		if (SDL_PollEvent(&event))
		{
			switch(event.type)
			{
				case SDL_QUIT:
					mainLoop = false;
					break;
			}
		}
		
		conversionSurface = SDL_ConvertSurface(vncSurface, mainSurface->format, 0);
		SDL_BlitSurface(vncSurface, NULL, mainSurface, &zeroRect);
		SDL_FreeSurface(conversionSurface);
		
		SDL_Flip(mainSurface);
		
	}
	
	SDL_Quit();
	
	fprintf(stderr, "done.\n");
	
	return 0;
}
